from geant4_pybind import *

#from particle import Particle
#particle = Particle.from_pdgid(pdgid)
#mass = particle.mass

class SteppingAction(G4UserSteppingAction):

  def __init__(self, app):
    super().__init__()
    self.app  = app
    self.data_buffer = app.data_buffer
    n = 1.35
    self.cerenkov_threshold_factor = 1./(1.-n**2)
    # E_th = mc^2 / sqrt(1-1/n^2),
    # where n = 1.35 for water, so 1/sqrt(1-1/n^2) = 1.4885
    #   for e+/e- E_th = 760 keV
    #   for mu    E_th = 157 MeV


  def UserSteppingAction(self, aStep):
    aTrack = aStep.GetTrack()
    particle = aTrack.GetDefinition()
    pdgid = particle.GetPDGEncoding()
    uid = aTrack.GetTrackID()
    parent_uid = aTrack.GetParentID()
    pre_step_point = aStep.GetPreStepPoint()
    post_step_point = aStep.GetPostStepPoint()
    position = post_step_point.GetPosition()/m
    time = post_step_point.GetGlobalTime()/ns
    momentum = post_step_point.GetMomentum()/GeV
    #Etot1 = post_step_point.GetTotalEnergy()/GeV
    Ekin1 = pre_step_point.GetKineticEnergy()/GeV
    Etot2 = post_step_point.GetTotalEnergy()/GeV
    #Ekin2 = post_step_point.GetKineticEnergy()/GeV
    # Save every track point 
    if Ekin1 > self.app.E_skip_min:
      self.data_buffer.tracks[uid].AddPoint([position.x, position.y, position.z, time, 
                                             momentum.x, momentum.y, momentum.z, Etot2] )



from g4camp.g4camp import g4camp
from time import time
import sys

n_events = 10

#app = g4camp(multithread=True, thread_num=16) # does not work faster
app = g4camp(multithread=False)
app.setMacro("electrons.mac")
app.setSkipMinMax(0.001, 0.05)
app.configure()

time0 = time()
for data in app.run(n_events):
    vertices = data.vertices
    tracks = data.tracks
    photon_cloud = data.photon_cloud
print(f"# Run time:  {(time()-time0):.2f} sec, {((time()-time0)/n_events):.2f} sec/event")



from geant4_pybind import *

class PrimaryGeneratorAction(G4VUserPrimaryGeneratorAction):

  def __init__(self):
    super().__init__()
    nofParticles = 1
    #self.fParticleGun = G4ParticleGun(nofParticles)
    self.fParticleGun = G4GeneralParticleSource()

    # default particle kinematic
    #particleDefinition = G4ParticleTable.GetParticleTable().FindParticle("mu-")
    #self.fParticleGun.SetParticleDefinition(particleDefinition)
    #self.fParticleGun.SetParticleMomentumDirection(G4ThreeVector(0, 0, 1))
    #self.fParticleGun.SetParticlePosition(G4ThreeVector(0, 0, 0))
    #self.fParticleGun.SetParticleEnergy(100*GeV)


  def GeneratePrimaries(self, anEvent):
    # This function is called at the begining of event
    self.fParticleGun.GeneratePrimaryVertex(anEvent)



import sys

import pkg_resources
available_packages = [pkg.key for pkg in pkg_resources.working_set]
if 'geant4-pybind' in available_packages:
    from geant4_pybind import *
else:
    sys.exit("Package 'geant4_pybind' is not available.")

from g4camp.DetectorConstruction import DetectorConstruction
from g4camp.CustomPhysicsList import CustomPhysicsList
from g4camp.ActionInitialization import ActionInitialization
from g4camp.DataBuffer import DataBuffer

class g4camp:

    def __init__(self, optics=True, multithread=False, thread_num=4):
        # By some reason multithread mode does not faster than serial mode
        # FIXME: number of threads can not be configured from here (why???)
        # For it is only configured by macro file
        #
        super().__init__()
        self.data_buffer = DataBuffer()
        #
        self.macro = "default.mac"
        self.optics = optics            # switch on/off optic physics
        self.ph_suppression_factor = 10 # must be integer
        self.skip_mode = 'fraction'     # 'fraction' (relative to E_init) or 'GeV' (absolute)
        self.skip_min = 1.e-3           # min energy to skip particle (wrt. E_init or in GeV)
        self.skip_max = 1.e-1           # max energy to skip particle (wrt. E_init or in GeV)
        self.random_seed = 1
        # the following varialbes are to be set later
        self.E_skip_min = None          # min particle energy to skip
        self.E_skip_max = None          # max particle energy to skip
        self.E_init = None              # initial kinetic energy in GeV
        #
        self.ph_counter = None
        #
        if multithread:
            self.runManager = G4RunManagerFactory.CreateRunManager(G4RunManagerType.MT, thread_num)
        else:
            self.runManager = G4RunManagerFactory.CreateRunManager(G4RunManagerType.Serial)
        self.detConstruction = DetectorConstruction()
        self.physList = CustomPhysicsList(optics=self.optics)
        self.actInit = ActionInitialization(self)
        self.runManager.SetUserInitialization(self.detConstruction)
        self.runManager.SetUserInitialization(self.physList)
        self.runManager.SetUserInitialization(self.actInit)


    def configure(self):
        #self.actInit.stackingAction.SetPhotonSuppressionFactor(1./self.ph_fraction)
        self.UImanager = G4UImanager.GetUIpointer()
        self.UImanager.ApplyCommand(f"/control/execute {self.macro}")
        G4Random.setTheSeed(self.random_seed)

    def setMacro(self, macro):
        self.macro = macro

    def setSkipMinMax(self, skip_min, skip_max):
        self.skip_min = skip_min
        self.skip_max = skip_max

    def setRandomSeed(self, val):
        self.random_seed = int(val)

    def setPhotonSuppressionFactor(self, val):
        self.ph_suppression_factor = float(val)

    def run(self, n_events):
        for i in range(n_events):
            self.runManager.BeamOn(1)
            yield self.data_buffer
